#include "paint.h"
#include "level.h"
#include "rules.h"

/* theLevel.Stage[row][column][top/bottom]
Rules for Compare:

MOVEABLE OBJECTS - User, Enemy, Helpers.
Everything else is IMMOVEABLE.

-1 = Game Over.
0 = The Moveable object just stopped on the field.
1 = The Moveable object is still in motion, remember to put a ontop of b.
5 = Character wins.

0 - Ground.
1 - Wall.
S - Main Character.
Y - Open Exit.
*/

Level CollectGem(Level theLevel, int RowA, int ColA, int RowB, int ColB)
{
 theLevel.Stage[RowB][ColB][0] = theLevel.Stage[RowA][ColA][0];
 theLevel.Stage[RowA][ColA][0] = theLevel.Stage[RowA][ColA][1];
 theLevel.Stage[RowA][ColA][1] = '0';
 theLevel.NumberOfGems = theLevel.NumberOfGems--;
  
 if(theLevel.NumberOfGems == 0)
 {
  theLevel = OpenExit(theLevel);
 }
 return theLevel; 	
}

Level PutBontopofA(Level theLevel, int RowA, int ColA, int RowB, int ColB)
{
 theLevel.Stage[RowB][ColB][1] = theLevel.Stage[RowA][ColA][0];
 theLevel.Stage[RowA][ColA][0] = theLevel.Stage[RowA][ColA][1];
 theLevel.Stage[RowA][ColA][1] = '0'; 	
 
 return theLevel;
}

Level Trap(Level theLevel, int RowA, int ColA, int RowB, int ColB)
{
 theLevel.Stage[RowB][ColB][0] = theLevel.Stage[RowB][ColB][1];
 theLevel.Stage[RowB][ColB][1] = '0';
 theLevel.Stage[RowA][ColA][0] = theLevel.Stage[RowA][ColA][1];
 theLevel.Stage[RowA][ColA][1] = '0';
 
 return theLevel;	
}

Level PushAround(Level theLevel, int RowA, int ColA, int RowB, int ColB)
{
 // PushDown b, move a ontop of b, pushup previous position of a; replace with 0 on the bottom.	
 theLevel.Stage[RowB][ColB][1] = theLevel.Stage[RowB][ColB][0];
 theLevel.Stage[RowB][ColB][0] = theLevel.Stage[RowA][ColA][0];
 theLevel.Stage[RowA][ColA][0] = theLevel.Stage[RowA][ColA][1];
 theLevel.Stage[RowA][ColA][1] = '0';
 
 return theLevel;
}

int Compare(char a, char b)
{
 // At the moment...only the character, walls, holes, and ground are in place...as well as the exit.	
	
 if(a == 'S')		// This checks the main character's reaction to all other possible combinations.
 {
  if(b == 'H')
  {
   return -1;		//Death by falling down a hole.  Good one.
  }	
  else if(b == 'T' || b == 'E')
  {
   return -2;		//Death by collision with something that cancels out both of them.	
  }
  else if(b == 'X' || b == '0')
  {
   return 1;		//Keep moving if it's just ground or a closed exit.
  }
  else if(b == '1' || b == 'B')
  {
   return 0;		//If it gets in your way, stop moving.
  }
  else if(b == 'G')
  {
   return 3; 		//If it's a gem, keep going but pick it up on the way.
  }
  else if(b == 'Y')
  {
   return 5;		//If it's the exit, you just finished the puzzle.
  }
 }
 else if( a == 'E')
 {
  if(b == 'S')
  {
   return -2;
  }
  if(b == 'H')
  {
   return 86;		//The enemy just died by falling down a hole.	
  }
  if(b == 'T')
  {
   return 87;
  }
  if(b == 'B' || b == '1' || b == 'G' || b == 'E')
  {
   return 0;	
  }
  else if(b == '0' || b == 'Y' || b == 'X')
  {
   return 1;		//Keep moving if it's just ground or a closed exit.
  }
 }
 else if(a == 'B')
 {
  if(b == 'S' || b == '1' || b == 'G' || b == 'E')
  {
   return 0;	
  }	
  if(b == 'H')
  {
   return 86;		//The enemy just died by falling down a hole.	
  }
  if(b == 'T')
  {
   return 87;
  } 	
  else if(b == '0' || b == 'Y' || b == 'X')
  {
   return 1;
  }
 }

 return 0;
}

Level Resolution(Level theLevel, int RowA, int ColA, int RowB, int ColB)
{
 int Resolve = 0;

 Resolve = Compare( theLevel.Stage[RowA][ColA][0], theLevel.Stage[RowB][ColB][0]);  

 if(Resolve == 86)
 {
  theLevel = PutBontopofA(theLevel, RowA, ColA, RowB, ColB); 	
 }
 if(Resolve == 87)
 {
  theLevel = Trap(theLevel, RowA, ColA, RowB, ColB);  		
 }
 if(Resolve == 1) 
 {
  theLevel = PushAround(theLevel, RowA, ColA, RowB, ColB);  
  theLevel.InMotion = 1;
 }
 else if(Resolve == 3)
 {
  theLevel = CollectGem(theLevel, RowA, ColA, RowB, ColB);	
  theLevel.InMotion = 1;
 }
 else if(Resolve == 5)
 {
  theLevel.Stage[0][0][1] = 'W';  //This is just a Done marker for testing. 	
  theLevel = PushAround(theLevel, RowA, ColA, RowB, ColB);
 }
 else if(Resolve == -1)
 {
  theLevel.Stage[0][0][1] = 'L';
  theLevel = PutBontopofA(theLevel, RowA, ColA, RowB, ColB);
 }
 else if(Resolve == -2)
 {
  theLevel.Stage[0][0][1] = 'L';
  theLevel = Trap(theLevel, RowA, ColA, RowB, ColB);  	
 }
    
 return theLevel;
}

Level MoveRight(Level theLevel)
{
 /*
   Check from top to bottom, compare array[i-1] to array[i].
   a on left, b on right.
   theLevel.Stage[row][column][top/bottom]
 */
 int i, j;
 
 theLevel.InMotion = 1;
  
 while(theLevel.InMotion == 1)
 {
  theLevel.InMotion = 0;
  	
  for(i = 24; i >= 0; i--)
  {
   for(j = 24; j > 0; j--)
   {
    theLevel = Resolution(theLevel, i, j-1, i, j);
   }	
  } 
  PrintLevel(theLevel);
  if(theLevel.Stage[0][0][1] == 'L' || theLevel.Stage[0][0][1] == 'W') theLevel.InMotion = 0;
 }

 return theLevel; 
}

Level MoveLeft(Level theLevel)
{
 /*
 Check from top to bottom, compare array[i] to array[i+1].
 b on left, a on right.
 */

 int i, j;
 
 theLevel.InMotion = 1;
  
 while(theLevel.InMotion == 1)
 {
  theLevel.InMotion = 0;
  	
  for(i = 0; i < 25; i++)
  {
   for(j = 0; j < 24; j++)
   {
    theLevel = Resolution(theLevel, i, j+1, i, j);
   }
  } 
  PrintLevel(theLevel);
  if(theLevel.Stage[0][0][1] == 'L' || theLevel.Stage[0][0][1] == 'W') theLevel.InMotion = 0;
 }

 return theLevel;  
}

Level MoveUp(Level theLevel)
{
 /*
 
 */
 int i, j;
 
 theLevel.InMotion = 1;
  
 while(theLevel.InMotion == 1)
 {
  theLevel.InMotion = 0;
  	
  for(i = 0; i < 25; i++)
  {
   for(j = 0; j < 24; j++)
   {
    theLevel = Resolution(theLevel, j+1, i, j, i);
   }	
  } 
  PrintLevel(theLevel);
  if(theLevel.Stage[0][0][1] == 'L' || theLevel.Stage[0][0][1] == 'W') theLevel.InMotion = 0;
 }

 return theLevel;   
}

Level MoveDown(Level theLevel)
{
 int i, j;
 
 theLevel.InMotion = 1;
  
 while(theLevel.InMotion == 1)
 {
  theLevel.InMotion = 0;
  	
  for(i = 24; i >= 0; i--)
  {
   for(j = 24; j > 0; j--)
   {
    theLevel = Resolution(theLevel, j-1, i, j, i);
   }	
  } 
  PrintLevel(theLevel);
  if(theLevel.Stage[0][0][1] == 'L' || theLevel.Stage[0][0][1] == 'W') theLevel.InMotion = 0;
 }

 return theLevel; 
}
